#!/usr/bin/python

import cgi
import ConfigParser
import re

## Set configuration file path
CONF_FILE = "/etc/nagios-oat-cgi/nagios-oat-cgi.conf"

""" 
####### Default output
https://.../nagios-oat-cgi/cgi-bin/nagstat2xml.cgi
####### Grouped by host
https://.../nagios-oat-cgi/cgi-bin/nagstat2xml.cgi?\
objs=hoststatus,servicestatus\
&hoststatus=all\
&servicestatus=service_description,plugin_output,current_state,last_hard_state\
&groupby=hoststatus
####### Grouped by service
https://.../nagios-oat-cgi/cgi-bin/nagstat2xml.cgi?\
objs=servicestatus\
&servicestatus=service_description,host_name,\
plugin_output,current_state,last_hard_state\
&groupby=servicestatus
"""

try:
    #yum install python-lxml
    from lxml import etree
    # print("running with lxml.etree")
except ImportError:
    try:
        # Python 2.5
        import xml.etree.cElementTree as etree
        # print("running with cElementTree on Python 2.5+")
    except ImportError:
        try:
            # Python 2.5
            import xml.etree.ElementTree as etree
            # print("running with ElementTree on Python 2.5+")
        except ImportError:
            try:
                # normal cElementTree install
                import cElementTree as etree
                # print("running with cElementTree")
            except ImportError:
                try:
                    # normal ElementTree install
                    import elementtree.ElementTree as etree
                    # print("running with ElementTree")
                except ImportError:
                    print("Failed to import ElementTree from any known place")

class NagStat2XML:
    statusFile = "/var/nagios/status.dat"
    objectsToGroup = {
            "hoststatus":"servicestatus",
            "servicestatus":"servicestatus"
            }
    groupByAllowed = {
            "servicestatus":"service_description", 
            "hoststatus":"host_name"
            }
    
    def __init__(self, statusFile, groupBy, keysToPrint, filterFields):
        self.statusFile = statusFile
        self.groupBy = groupBy
        self.keysToPrint = keysToPrint
        self.filterFields = filterFields
        
        self.groups = dict()
        self.groupsObjs = dict()
        self.nagios = None
        
    def getSubnodeValue(self, node):
        if node == None:
            return None
        value = node.get("value")
        if not value:
            value = node.text
        return value
        
    def isObjectPrintable(self, node):
        if node != None:
            return self.isObjectNamePrintable(node.tag)
        return False
        
    def isObjectNamePrintable(self, name):
        return self.keysToPrint.has_key(name)
        
    def createObjectSubnode(self, line, currentObj):
        pieces = [p.strip() for p in line.split("=",1)]
        if (pieces[1] and
              (self.keysToPrint[currentObj.tag]=="all" 
               or (isinstance(self.keysToPrint[currentObj.tag], list) 
                   and pieces[0] in self.keysToPrint[currentObj.tag]) 
               or self.groupByAllowed.get(self.groupBy, "") == pieces[0]
               or self.filterFields.get(currentObj.tag, dict()).
                                                has_key(pieces[0]))):
            if re.compile("\d+").match(pieces[1]):
                newNode = etree.Element(pieces[0], value=pieces[1])
            else:
                newNode = etree.Element(pieces[0])
                newNode.text = pieces[1]
            return newNode
        else:
            return None
    
    def createObjectNode(self, line):
        name = line.split(" ")[0]
        if self.isObjectNamePrintable(name):
            return etree.Element(name)
        else:
            return None
            
    def groupByActivated(self):
        return self.groupByAllowed.get(self.groupBy, "")
    
    def groupObject(self, node):
        if node == None:
            return
        subnode = node.find(self.groupByAllowed[self.groupBy])
        subnodevalue = self.getSubnodeValue(subnode)
        if subnodevalue:
            if node.tag == "hoststatus" and self.groupBy == node.tag:
                self.nagios.append(node)
                self.groups[subnodevalue] = node
            elif (node.tag == "servicestatus" 
                  and self.groupBy == node.tag 
                  and (not self.groups.has_key(subnodevalue))):
                tmpNode = etree.Element("service", description=subnodevalue)
                self.nagios.append(tmpNode)
                self.groups[subnodevalue] = tmpNode
            if self.objectsToGroup.get(self.groupBy,"") == node.tag:
                if not self.groupsObjs.has_key(subnodevalue):
                    self.groupsObjs[subnodevalue] = list()
                self.groupsObjs[subnodevalue].append(node)
    
    def appendGroupedObjects(self):
        if not self.groups:
            return
        for k, v in self.groups.items():
            if self.groupsObjs.has_key(k):
                for vv in self.groupsObjs[k]:
                    v.append(vv)
    
    def matchFilter(self, node):
        if (not self.filterFields) or node == None:
            return True
        for k, v in self.filterFields.get(node.tag, dict()).items():
            subnode = node.find(k)
            if subnode == None:
                continue
            elif self.getSubnodeValue(subnode) != v:
                return False
        return True
        
    def isObjectToGroup(self, node):
        if node == None:
            return False
        return (self.groupByActivated() and
               (node.tag == self.groupBy or 
                self.objectsToGroup.get(self.groupBy, "") == node.tag))
        
    def removeExtraFields(self, node):
        if node == None:
            return
        # Removing filtering extra fields
        for f in self.filterFields.get(node.tag, dict()):
            if f not in self.keysToPrint.get(node.tag, []):
                snode = node.find(f)
                if snode != None:
                    node.remove(snode)
        # Removing grouping extra fields
        elmName = self.groupByAllowed.get(self.groupBy, None)
        if (elmName and self.keysToPrint[node.tag] != "all" and
                isinstance(self.keysToPrint[node.tag], list) and
                (elmName not in self.keysToPrint.get(node.tag, []))):
            elm = node.find(elmName)
            if elm != None:
                node.remove(elm)
    
    def toXML(self):
        self.nagios = etree.Element("nagios")
        try:
            sfile = open(self.statusFile, 'r')
        except IOError:
            self.nagios.text = "file doesn't exist"
            return self.nagios
        
        currentObj = None
        for line in sfile:
            if re.match("^\s*(\#.*)?$", line):          # comment or empty line
                continue
            elif line.strip() == "}":                   # Object closed
                if (self.isObjectPrintable(currentObj) and 
                   self.matchFilter(currentObj)):
                    if self.isObjectToGroup(currentObj):
                        self.groupObject(currentObj)
                    else:
                        self.nagios.append(currentObj)
                    self.removeExtraFields(currentObj)
            elif re.match("^\s*[a-z]+ {\s*$", line):    # Object opened
                currentObj = self.createObjectNode(line)
            elif self.isObjectPrintable(currentObj):    # Object attribute
                snode = self.createObjectSubnode(line, currentObj)
                if snode != None:
                    currentObj.append(snode)
        sfile.close()                                   # Closing file
        if self.groupByActivated():
            self.appendGroupedObjects()
                
        return self.nagios
       
# Missing in Python 2.4
def startswith(string, t):
    for i in t:
        if string.startswith(i):
            return True
    return False
    
def readGroupBy(form, default_value):
    groupBy = form.getfirst("groupby", "")
    if re.match("[a-zA-Z]+", groupBy):
        return groupBy
    else:
        return default_value
        
def readObjs(form, defaultKeysToPrint, restrictKeysToPrint):
    objs = form.getfirst("objs", "")
    objsok = list()
    if(objs == "all"):
        objsok = restrictKeysToPrint.keys()
    elif re.match("[a-z,]+", objs):
        objsok = objs.split(",")
    if not objsok:
        objsok = defaultKeysToPrint.keys()
    return objsok
    
def readKeysToPrint(form, objsok, defaultKeysToPrint, restrictKeysToPrint):
    keysToPrint = dict()
    for o in objsok:
        if not restrictKeysToPrint.has_key(o):
            continue
        ks = form.getfirst(o, "")
        keysToPrint[o] = list()
        if ks == "all":
            keysToPrint[o] = restrictKeysToPrint[o]
        elif re.match("[a-z_,]+", ks):
            keysToPrint[o] = ks.split(",")
        if not keysToPrint[o]:
            keysToPrint[o] = defaultKeysToPrint[o]
    return keysToPrint
    
def checkKeysRestrictions(keysToPrint, restrictKeysToPrint):
    keysToPrintOk = dict()
    for k, v in keysToPrint.items():
        if v == "all":
            keysOk = v
        else:
            keysOk = list()
            for key in v:
                if (restrictKeysToPrint[k] == "all" or
                      ( isinstance(restrictKeysToPrint[k], list) and 
                        key in restrictKeysToPrint[k])):
                    keysOk.append(key)
        keysToPrintOk[k] = keysOk
    return keysToPrintOk

def readFilterFields(form, restrictFilterFields):
    filterFields = dict()
    for k, o in restrictFilterFields.items():
        fi = dict()
        for field in o:
            fs = form.getfirst("%s.%s" % (k, field), "")
            if fs:
                fi[field] = fs
        filterFields[k] = fi
    return filterFields

def readParam(config):
    form = cgi.FieldStorage()
    
    groupByOk = readGroupBy(form, config.main['group_by'])
    objects = readObjs(form, config.defaultKeysToPrint, config.restrictKeysToPrint)
    keysToPrint = readKeysToPrint(form, objects, config.defaultKeysToPrint, config.restrictKeysToPrint)
    keysToPrintOk = checkKeysRestrictions(keysToPrint, config.restrictKeysToPrint)
    filterFieldsOk = readFilterFields(form, config.restrictFilterFields)
    
    return (groupByOk, keysToPrintOk, filterFieldsOk)
    
def loadConfigFile(conf_file):
    config = ConfigParser.ConfigParser()
    config.read(conf_file)
    cfg = Config()
    cfg.main = dict(config.items('main'))
    cfg.defaultKeysToPrint = dict([(x, y.split(',')) for x,y in config.items('defaultKeysToPrint')])
    cfg.restrictKeysToPrint = dict()
    for k, v in dict([(x, y.split(',')) for x,y in config.items('restrictKeysToPrint')]).items():
        if 'all' in v:
            cfg.restrictKeysToPrint[k] = 'all'
    cfg.restrictFilterFields = dict([(x, y.split(',')) for x,y in config.items('restrictFilterFields')])
    return cfg
    
def enableWebDebug(val):
    if val == '1':
        import cgitb; cgitb.enable()
    
class Config:
    pass

if __name__ == '__main__':
    config = loadConfigFile(CONF_FILE)
    enableWebDebug(config.main['web_debug'])
    (gb, ktp, ff) = readParam(config)
    app = NagStat2XML(config.main['status_file'], gb, ktp, ff)
    doc = app.toXML()
    print "Content-type: text/xml"
    print
    print etree.tostring(doc)
